//--------------------------------------------------------------------------
// Animated flame for Adafruit Pro Trinket.  Uses the following parts:
//   - Pro Trinket microcontroller (adafruit.com/product/2010 or 2000)
//     (#2010 = 3V/12MHz for longest battery life, but 5V/16MHz works OK)
//   - Charlieplex LED Matrix Driver (2946)
//   - Charlieplex LED Matrix (2947, 2948, 2972, 2973 or 2974)
//   - 350 mAh LiPoly battery (2750)
//   - LiPoly backpack (2124)
//   - SPDT Slide Switch (805)
//
// This is NOT good "learn from" code for the IS31FL3731; it is "squeeze
// every last byte from the Pro Trinket" code.  If you're starting out,
// download the Adafruit_IS31FL3731 and Adafruit_GFX libraries, which
// provide functions for drawing pixels, lines, etc.  This sketch also
// uses some ATmega-specific tricks and will not run as-is on other chips.
//--------------------------------------------------------------------------

#include <Wire.h>           // For I2C communication
#include <Adafruit_GFX.h>
#include <Adafruit_IS31FL3731.h>

#include "data.h"

Adafruit_IS31FL3731 ledmatrix = Adafruit_IS31FL3731();

// The lookup table to make the brightness changes be more visible
//uint8_t sweep[] = {1, 2, 3, 4, 6, 8, 10, 15, 20, 30, 40, 60, 60, 40, 30, 20, 15, 10, 8, 6, 4, 3, 2, 1};

const uint8_t *ptr  = anim; // Current pointer into animation data

void setup() {
  // swirldemo
  Serial.begin(9600);
  Serial.println("ISSI swirl test");

  if (! ledmatrix.begin()) {
    Serial.println("IS31 not found");
    while (1);
  }
  Serial.println("IS31 found!");
}

void loop() {
  uint8_t  a, x1, y1, x2, y2, x, y;

  // Then render NEXT frame.  Start by getting bounding rect for new frame:
  a = pgm_read_byte(ptr++);     // New frame X1/Y1
  if(a >= 0x90) {               // EOD marker? (valid X1 never exceeds 8)
    ptr = anim;                 // Reset animation data pointer to start
    a   = pgm_read_byte(ptr++); // and take first value
  }
  x1 = a >> 4;                  // X1 = high 4 bits
  y1 = a & 0x0F;                // Y1 = low 4 bits
  a  = pgm_read_byte(ptr++);    // New frame X2/Y2
  x2 = a >> 4;                  // X2 = high 4 bits
  y2 = a & 0x0F;                // Y2 = low 4 bits

  for(x=x1; x<=x2; x++) { // Column-major
    for(y=y1; y<=y2; y++) {
      ledmatrix.drawPixel(y, x, pgm_read_byte(ptr++));
    }
  }

  // TEST
  // animate over all the pixels, and set the brightness from the sweep table
  /*
  for (uint8_t incr = 0; incr < 24; incr++) {
    for (uint8_t x = 0; x < 16; x++) {
      for (uint8_t y = 0; y < 9; y++) {
        //ledmatrix.drawPixel(x, y, sweep[(x+y+incr)%24]);
        //ledmatrix.drawPixel(x, y, random(30));
        ledmatrix.drawPixel(x, y, pgm_read_byte(ptr++));
      }
    }
  }
  */
  //delay(20);

}
